@@ -1,17 +1,22 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2003-2008 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  * @endcopyright
  */
@@ -42,7 +47,7 @@ public interface SVNClientInterface
      * @return The name of the working copy's administrative
      * directory, which is usually <code>.svn</code>.
      * @see <a
-     * href="http://svn.collab.net/repos/svn/trunk/notes/asp-dot-net-hack.txt">
+     * href="http://svn.apache.org/repos/asf/subversion/trunk/notes/asp-dot-net-hack.txt">
      * Instructions on changing this as a work-around for the behavior of
      * ASP.Net on Windows.</a>
      * @since 1.3
@@ -149,7 +154,7 @@ public interface SVNClientInterface
             throws ClientException;
 
     /**
-     * Lists the directory entries of an url on the server.
+     * Lists the directory entries of a url on the server.
      * @param url       the url to list
      * @param revision  the revision to list
      * @param recurse   recurse into subdirectories
@@ -162,7 +167,7 @@ public interface SVNClientInterface
             throws ClientException;
 
     /**
-     * Lists the directory entries of an url on the server.
+     * Lists the directory entries of a url on the server.
      * @param url       the url to list
      * @param revision  the revision to list
      * @param pegRevision the revision to interpret url
@@ -177,7 +182,7 @@ public interface SVNClientInterface
             throws ClientException;
 
     /**
-     * Lists the directory entries of an url on the server.
+     * Lists the directory entries of a url on the server.
      * @param url             the url to list
      * @param revision        the revision to list
      * @param pegRevision     the revision to interpret url
@@ -326,7 +331,7 @@ public interface SVNClientInterface
      * Retrieve the log messages for an item.
      * @param path          path or url to get the log message for.
      * @param pegRevision   revision to interpret path
-     * @param revisionRanges an array of revision ranges to show
+     * @param ranges        an array of revision ranges to show
      * @param stopOnCopy    do not continue on copy operations
      * @param discoverPath  returns the paths of the changed items in the
      *                      returned objects
@@ -431,7 +436,7 @@ public interface SVNClientInterface
 
     /**
      * Sets the commit message handler. This allows more complex commit message
-     * with the list of the elements to be commited as input.
+     * with the list of the elements to be committed as input.
      * @param messageHandler    callback for entering commit messages
      *                          if this is set the message parameter is ignored.
      * @since 1.0
@@ -651,6 +656,29 @@ public interface SVNClientInterface
                 String[] changelists, Map revpropTable)
             throws ClientException;
 
+    /**
+     * Copy versioned paths with the history preserved.
+     *
+     * @param sources A list of <code>CopySource</code> objects.
+     * @param destPath Destination path or URL.
+     * @param message Commit message.  May be <code>null</code> if
+     * <code>destPath</code> is not a URL.
+     * @param copyAsChild Whether to copy <code>srcPaths</code> as
+     * children of <code>destPath</code>.
+     * @param makeParents Whether to create intermediate parents
+     * @param ignoreExternals Whether or not to process external definitions
+     *                        as part of this operation.
+     * @param revpropTable A string-to-string mapping of revision properties
+     *                     to values which will be set if this operation
+     *                     results in a commit.
+     * @throws ClientException If the copy operation fails.
+     * @since 1.7
+     */
+    void copy(CopySource[] sources, String destPath, String message,
+              boolean copyAsChild, boolean makeParents,
+              boolean ignoreExternals, Map revpropTable)
+            throws ClientException;
+
     /**
      * Copy versioned paths with the history preserved.
      *
@@ -666,6 +694,8 @@ public interface SVNClientInterface
      *                     results in a commit.
      * @throws ClientException If the copy operation fails.
      * @since 1.5
+     * @deprecated Use {@link #copy(CopySource[], String, String, boolean,
+     *                              boolean, boolean, Map)} instead.
      */
     void copy(CopySource[] sources, String destPath, String message,
               boolean copyAsChild, boolean makeParents, Map revpropTable)
@@ -677,11 +707,11 @@ public interface SVNClientInterface
      *
      * @param srcPath   source path or url
      * @param destPath  destination path or url
-     * @param message   commit message if destPath is an url
+     * @param message   commit message if destPath is a url
      * @param revision  source revision
      * @throws ClientException
      * @deprecated Use {@link #copy(CopySource[], String, String, boolean,
-     *                              boolean)} instead.
+     *                              boolean, boolean, Map)} instead.
      */
     void copy(String srcPath, String destPath, String message,
               Revision revision) throws ClientException;
@@ -724,7 +754,7 @@ public interface SVNClientInterface
      *
      * @param srcPath   source path or url
      * @param destPath  destination path or url
-     * @param message   commit message if destPath is an url
+     * @param message   commit message if destPath is a url
      * @param force     even with local modifications.
      * @throws ClientException
      * @deprecated Use {@link #move(String[], String, String, boolean, boolean,
@@ -1063,7 +1093,29 @@ public interface SVNClientInterface
 
     /**
      * Retrieve either merged or eligible-to-be-merged revisions.
-     * @param kind                   kind of revisions to receive
+     * @param kind                   kind of revisions to receive:
+     * See {@see org.apache.subversion.javahl.MergeinfoLogKind}.
+     * @param pathOrUrl              target of merge
+     * @param pegRevision            peg rev for pathOrUrl
+     * @param mergeSourceUrl         the source of the merge
+     * @param srcPegRevision         peg rev for mergeSourceUrl
+     * @param discoverChangedPaths   return paths of changed items
+     * @param depth                  the depth to recurse to
+     * @param revProps               the revprops to retrieve
+     * @param callback               the object to receive the log messages
+     * @since 1.7
+     */
+    void getMergeinfoLog(int kind, String pathOrUrl,
+                         Revision pegRevision, String mergeSourceUrl,
+                         Revision srcPegRevision, boolean discoverChangedPaths,
+                         int depth, String[] revProps,
+                         LogMessageCallback callback)
+        throws ClientException;
+
+    /**
+     * Retrieve either merged or eligible-to-be-merged revisions.
+     * @param kind                   kind of revisions to receive:
+     * See {@see org.apache.subversion.javahl.MergeinfoLogKind}.
      * @param pathOrUrl              target of merge
      * @param pegRevision            peg rev for pathOrUrl
      * @param mergeSourceUrl         the source of the merge
@@ -1071,12 +1123,15 @@ public interface SVNClientInterface
      * @param discoverChangedPaths   return paths of changed items
      * @param revProps               the revprops to retrieve
      * @param callback               the object to receive the log messages
+     * @deprecated Use {@link #getMergeinfoLog(int, String, Revision, String,
+     *                                         Revision, boolean, int,
+     *                                         String[], LogMessageCallback)}
      * @since 1.5
      */
     void getMergeinfoLog(int kind, String pathOrUrl,
                          Revision pegRevision, String mergeSourceUrl,
                          Revision srcPegRevision, boolean discoverChangedPaths,
-                         String[] revprops, LogMessageCallback callback)
+                         String[] revProps, LogMessageCallback callback)
         throws ClientException;
 
     /**
@@ -1131,7 +1186,33 @@ public interface SVNClientInterface
      * @param ignoreAncestry ignore if files are not related
      * @param noDiffDeleted no output on deleted files
      * @param force         diff even on binary files
+     * @param copiesAsAdds  if set, copied files will be shown in their
+     *                      entirety, not as diffs from their sources
      * @throws ClientException
+     * @since 1.7
+     */
+    void diff(String target1, Revision revision1, String target2,
+              Revision revision2, String relativeToDir, String outFileName,
+              int depth, String[] changelists, boolean ignoreAncestry,
+              boolean noDiffDeleted, boolean force, boolean copiesAsAdds)
+            throws ClientException;
+
+    /**
+     * Display the differences between two paths
+     * @param target1       first path or url
+     * @param revision1     first revision
+     * @param target2       second path or url
+     * @param revision2     second revision
+     * @param relativeToDir index path is relative to this path
+     * @param outFileName   file name where difference are written
+     * @param depth         how deep to traverse into subdirectories
+     * @param ignoreAncestry ignore if files are not related
+     * @param noDiffDeleted no output on deleted files
+     * @param force         diff even on binary files
+     * @throws ClientException
+     * @deprecated Use {@link #diff(String, Revision, String, Revision,
+     *                              String, String, int, boolean, boolean,
+     *                              boolean, boolean)} instead.
      * @since 1.5
      */
     void diff(String target1, Revision revision1, String target2,
@@ -1175,7 +1256,34 @@ public interface SVNClientInterface
      * @param ignoreAncestry ignore if files are not related
      * @param noDiffDeleted no output on deleted files
      * @param force         diff even on binary files
+     * @param copiesAsAdds  if set, copied files will be shown in their
+     *                      entirety, not as diffs from their sources
      * @throws ClientException
+     * @since 1.7
+     */
+    void diff(String target, Revision pegRevision, Revision startRevision,
+              Revision endRevision, String relativeToDir, String outFileName,
+              int depth, String[] changelists, boolean ignoreAncestry,
+              boolean noDiffDeleted, boolean force, boolean copiesAsAdds)
+            throws ClientException;
+
+    /**
+     * Display the differences between two paths.
+     * @param target        path or url
+     * @param pegRevision   revision tointerpret target
+     * @param startRevision first Revision to compare
+     * @param endRevision   second Revision to compare
+     * @param relativeToDir index path is relative to this path
+     * @param outFileName   file name where difference are written
+     * @param depth         how deep to traverse into subdirectories
+     * @param changelists  if non-null, filter paths using changelists
+     * @param ignoreAncestry ignore if files are not related
+     * @param noDiffDeleted no output on deleted files
+     * @param force         diff even on binary files
+     * @throws ClientException
+     * @deprecated Use {@link #diff(String, Revision, Revision, Revision,
+     *                              String, String, int, boolean, boolean,
+     *                              boolean, boolean)} instead.
      * @since 1.5
      */
     void diff(String target, Revision pegRevision, Revision startRevision,
@@ -1222,9 +1330,9 @@ public interface SVNClientInterface
      * #diffSummarize(String, Revision, String, Revision, int,
      * boolean, DiffSummaryReceiver)}, using <code>path</code> for
      * both of that method's targets.
-     * @param startRevision Beginning of range for comparsion of
+     * @param startRevision Beginning of range for comparison of
      * <code>target</code>.
-     * @param endRevision End of range for comparsion of
+     * @param endRevision End of range for comparison of
      * <code>target</code>.
      * @param depth how deep to recurse.
      * @param changelists  if non-null, filter paths using changelists
@@ -1692,14 +1800,36 @@ public interface SVNClientInterface
      * @param callback      callback to receive the file content and the other
      *                      information
      * @throws ClientException
+     * @deprecated Use {@link #blame(String, Revision, Revision, Revision,
+     *                               boolean, boolean, BlameCallback3)}
+     *                               instead.
      * @since 1.5
      */
-
     void blame(String path, Revision pegRevision, Revision revisionStart,
                Revision revisionEnd, boolean ignoreMimeType,
                boolean includeMergedRevisions,
                BlameCallback2 callback) throws ClientException;
 
+    /**
+     * Retrieve the content together with the author, the revision and the date
+     * of the last change of each line
+     * @param path          the path
+     * @param pegRevision   the revision to interpret the path
+     * @param revisionStart the first revision to show
+     * @param revisionEnd   the last revision to show
+     * @param ignoreMimeType whether or not to ignore the mime-type
+     * @param includeMergedRevisions whether or not to include extra merge
+     *                      information
+     * @param callback      callback to receive the file content and the other
+     *                      information
+     * @throws ClientException
+     * @since 1.7
+     */
+    void blame(String path, Revision pegRevision, Revision revisionStart,
+               Revision revisionEnd, boolean ignoreMimeType,
+               boolean includeMergedRevisions,
+               BlameCallback3 callback) throws ClientException;
+
     /**
      * Set directory for the configuration information, taking the
      * usual steps to ensure that Subversion's config file templates
@@ -1758,7 +1888,7 @@ public interface SVNClientInterface
      * @param changelists changelists to filter by
      * @since 1.5
      */
-    void removeFromChangelists(String[] paths, int depth, String[] changelist)
+    void removeFromChangelists(String[] paths, int depth, String[] changelists)
             throws ClientException;
 
     /**
@@ -1816,7 +1946,7 @@ public interface SVNClientInterface
      * @param pegRevision   the revision to interpret pathOrUrl
      * @param depth         the depth to recurse
      * @param changelists   if non-null, filter paths using changelists
-     * @param callback      a callback to receive the infos retreived
+     * @param callback      a callback to receive the infos retrieved
      * @since 1.5
      */
     void info2(String pathOrUrl, Revision revision, Revision pegRevision,
@@ -1834,4 +1964,13 @@ public interface SVNClientInterface
      */
     String getVersionInfo(String path, String trailUrl, boolean lastChanged)
             throws ClientException;
+
+    /**
+     * Recursively upgrade a working copy to a new metadata storage format.
+     * @param path                  path of the working copy
+     * @throws ClientException
+     * @since 1.7
+     */
+    void upgrade(String path)
+            throws ClientException;
 }
