@@ -207,8 +207,7 @@ svn_auth_get_parameter(svn_auth_baton_t *auth_baton,
   value = svn_hash_gets(auth_baton->slave_parameters, name);
 
   if (value)
-    return (value == &auth_NULL) ? NULL
-                                : value;
+    return (value == &auth_NULL ? NULL : value);
 
   return svn_hash_gets(auth_baton->parameters, name);
 }
@@ -394,13 +393,11 @@ svn_auth_save_credentials(svn_auth_iterstate_t *state,
   svn_auth_provider_object_t *provider;
   svn_boolean_t save_succeeded = FALSE;
   const char *no_auth_cache;
-  svn_auth_baton_t *auth_baton;
   void *creds;
 
   if (! state || state->table->providers->nelts <= state->provider_idx)
     return SVN_NO_ERROR;
 
-  auth_baton = state->auth_baton;
   creds = svn_hash_gets(state->auth_baton->creds_cache, state->cache_key);
   if (! creds)
     return SVN_NO_ERROR;
@@ -505,7 +502,7 @@ svn_auth_get_platform_specific_provider(svn_auth_provider_object_t **provider,
   if (apr_strnatcmp(provider_name, "gnome_keyring") == 0 ||
       apr_strnatcmp(provider_name, "kwallet") == 0)
     {
-#if defined(SVN_HAVE_GNOME_KEYRING) || defined(SVN_HAVE_KWALLET)
+#if defined(SVN_HAVE_GNOME_KEYRING) || defined(SVN_HAVE_KWALLET) || defined (SVN_HAVE_LIBSECRET)
       apr_dso_handle_t *dso;
       apr_dso_handle_sym_t provider_function_symbol, version_function_symbol;
       const char *library_label, *library_name;
@@ -770,12 +767,10 @@ svn_auth__make_session_auth(svn_auth_baton_t **session_auth_baton,
   * "store-auth-creds = yes" -- they'll get the expected behaviour.
   */
 
-  if (svn_auth_get_parameter(ab,
-                              SVN_AUTH_PARAM_DONT_STORE_PASSWORDS) != NULL)
+  if (svn_auth_get_parameter(ab, SVN_AUTH_PARAM_DONT_STORE_PASSWORDS) != NULL)
     store_passwords = FALSE;
 
-  if (svn_auth_get_parameter(ab,
-                              SVN_AUTH_PARAM_NO_AUTH_CACHE) != NULL)
+  if (svn_auth_get_parameter(ab, SVN_AUTH_PARAM_NO_AUTH_CACHE) != NULL)
     store_auth_creds = FALSE;
 
   /* All the svn_auth_set_parameter() calls below this not only affect the
@@ -860,8 +855,7 @@ svn_auth__make_session_auth(svn_auth_baton_t **session_auth_baton,
 
   /* Save auth caching parameters in the auth parameter hash. */
   if (! store_passwords)
-    svn_auth_set_parameter(ab,
-                           SVN_AUTH_PARAM_DONT_STORE_PASSWORDS, "");
+    svn_auth_set_parameter(ab, SVN_AUTH_PARAM_DONT_STORE_PASSWORDS, "");
 
   svn_auth_set_parameter(ab,
                          SVN_AUTH_PARAM_STORE_PLAINTEXT_PASSWORDS,
@@ -877,8 +871,7 @@ svn_auth__make_session_auth(svn_auth_baton_t **session_auth_baton,
                          store_pp_plaintext);
 
   if (! store_auth_creds)
-    svn_auth_set_parameter(ab,
-                           SVN_AUTH_PARAM_NO_AUTH_CACHE, "");
+    svn_auth_set_parameter(ab, SVN_AUTH_PARAM_NO_AUTH_CACHE, "");
 
   if (server_group)
     svn_auth_set_parameter(ab,
